#____________________________________________________________________________
# Creating series of state and change values, and RAM matrices

	val <- c(	round(seq(1000,2000,length.out=steps+2),0),
				round(seq(2000,3000,length.out=steps+2),0)[-1],
				round(seq(3000,4000,length.out=steps+2),0)[-1])
	dt <- ObservationInterval/(steps+1)

	manifest <- c(	paste("MSTx",c(0,6,12,18),sep=""),
		paste("MSTy",c(0,6,12,18),sep=""))
	latent <- c(	c(paste("der0_",val,"sta_x",sep=""),
		c("der1_0000_x",paste("der1_",val,"_x",sep="")),"traitINT_x"),	
		c(paste("der0_",val,"sta_y",sep=""),
		c("der1_0000_y",paste("der1_",val,"_y",sep="")),"traitINT_y"))
	objects <- c(latent,manifest)

	Amatrix <- mxMatrix(type="Full",
		nrow=length(objects),ncol=length(objects),
		name="A",dimnames=list(objects,objects))
	Smatrix <- mxMatrix(type="Symm",
		nrow=length(objects),ncol=length(objects),
		name="S",dimnames=list(objects,objects))
	Fmatrix <- mxMatrix(type="Full",
		nrow=length(manifest),ncol=length(objects),
		name="F",dimnames=list(manifest,objects))
	diag(Fmatrix@values[,(length(latent)+1):length(objects)]) <- 1
	Imatrix <- mxMatrix(type="Iden",ncol=length(objects),name="I")
	Mmatrix <- mxMatrix("Full",
		nrow=1,ncol=length(objects),name="M",dimnames=list(NA,objects))
	
#____________________________________________________________________________
# Add random intercept paths to A matrix
	Amatrix@values[which(substr(objects,1,4)=="MSTx"),
		which(objects=="traitINT_x")] <- 1
	Amatrix@values[which(substr(objects,1,4)=="MSTy"),
		which(objects=="traitINT_y")] <- 1

#____________________________________________________________________________
# Set subsequent state equal to prior state
		
	sel <- which((substr(objects,1,4)=="der0")&
		(substr(objects,10,12)=="sta")&(substr(objects,14,14)=="x"))
	diag(Amatrix@values[sel[-1],sel[-length(sel)]]) <- 1
	
	sel <- which((substr(objects,1,4)=="der0")&
		(substr(objects,10,12)=="sta")&(substr(objects,14,14)=="y"))
	diag(Amatrix@values[sel[-1],sel[-length(sel)]]) <- 1

#____________________________________________________________________________
# Regress subsequent state onto prior velocity
	
	sel1 <- which((substr(objects,1,4)=="der0")&
		(substr(objects,10,12)=="sta")&(substr(objects,14,14)=="x"))
	sel2 <- which((substr(objects,1,4)=="der1")&(substr(objects,11,11)=="x"))
	diag(Amatrix@values[sel1,sel2[-length(sel2)]]) <- dt
	
	sel1 <- which((substr(objects,1,4)=="der0")&
		(substr(objects,10,12)=="sta")&(substr(objects,14,14)=="y"))
	sel2 <- which((substr(objects,1,4)=="der1")&(substr(objects,11,11)=="y"))
	diag(Amatrix@values[sel1,sel2[-length(sel2)]]) <- dt

#____________________________________________________________________________
# Add paths from level to velocity within variables
	
	sel1 <- which((substr(objects,1,4)=="der0")&
		(substr(objects,10,12)=="sta")&(substr(objects,14,14)=="x"))
	sel2 <- which((substr(objects,1,4)=="der1")&(substr(objects,11,11)=="x"))
	diag(Amatrix@values[sel2[-1],sel1]) <- (-.01)
	diag(Amatrix@labels[sel2[-1],sel1]) <- "Ax"
	diag(Amatrix@free[sel2[-1],sel1]) <- TRUE

	sel1 <- which((substr(objects,1,4)=="der0")&
		(substr(objects,10,12)=="sta")&(substr(objects,14,14)=="y"))
	sel2 <- which((substr(objects,1,4)=="der1")&(substr(objects,11,11)=="y"))
	diag(Amatrix@values[sel2[-1],sel1]) <- (-.01)
	diag(Amatrix@labels[sel2[-1],sel1]) <- "Ay"
	diag(Amatrix@free[sel2[-1],sel1]) <- TRUE

#____________________________________________________________________________
# Estimate variance of stochastic perturbations independent of dt

	diag(Smatrix@labels[which((substr(objects,1,4)=="der1")&
		(substr(objects,11,11)=="x")),which((substr(objects,1,4)=="der1")&
		(substr(objects,11,11)=="y"))]) <- "Rescale[2,1]"
	diag(Smatrix@labels[which((substr(objects,1,4)=="der1")&
		(substr(objects,11,11)=="y")),which((substr(objects,1,4)=="der1")&
		(substr(objects,11,11)=="x"))]) <- "Rescale[2,1]"
				
	sel <- which((substr(objects,1,4)=="der1")&(substr(objects,11,11)=="x"))
	diag(Smatrix@labels[sel,sel]) <- "Rescale[1,1]"
	
	sel <- which((substr(objects,1,4)=="der1")&(substr(objects,11,11)=="y"))
	diag(Smatrix@labels[sel,sel]) <- "Rescale[2,2]"
	
	StochasticErr <- mxMatrix(type = "Full", nrow = 2, ncol = 2,name = "Q", 
		free = c(T,T,F,T),values = c(1,0,0,1),
		labels = c("VarD1x","CovD1xy",NA,"VarD1y"),
		lbound = c(0,NA,NA,0))			
	dtMat <- mxMatrix(type = "Full", nrow = 2, ncol = 2, name = "dtMat", 
		free = FALSE, value = dt)  
	NewAlg <- mxAlgebra((Q)/(dtMat), name = "Rescale")

#____________________________________________________________________________
# Add variance and covariance to random intercepts		

		Smatrix@values[which(objects=="traitINT_x"),
			which(objects=="traitINT_x")] <- 10
		Smatrix@labels[which(objects=="traitINT_x"),
			which(objects=="traitINT_x")] <- "VAR_traitINT_x"
		Smatrix@free[which(objects=="traitINT_x"),
			which(objects=="traitINT_x")] <- TRUE

		Smatrix@values[which(objects=="traitINT_y"),
			which(objects=="traitINT_y")] <- 10
		Smatrix@labels[which(objects=="traitINT_y"),
			which(objects=="traitINT_y")] <- "VAR_traitINT_y"
		Smatrix@free[which(objects=="traitINT_y"),
			which(objects=="traitINT_y")] <- TRUE

		Smatrix@labels[which(objects=="traitINT_x"),
			which(objects=="traitINT_y")] <- "COV_traitINTx_INTy"
		Smatrix@free[which(objects=="traitINT_x"),
			which(objects=="traitINT_y")] <- TRUE
		Smatrix@labels[which(objects=="traitINT_y"),
			which(objects=="traitINT_x")] <- "COV_traitINTx_INTy"
		Smatrix@free[which(objects=="traitINT_y"),
			which(objects=="traitINT_x")] <- TRUE

#____________________________________________________________________________
# Relate time varying states of X to manifest variables

	val <- c(1000,2000,3000,4000)
	val1 <- c(6,12,18,24)
	for(i in 1:length(val)) {
		sel <- which((substr(objects,1,4)=="MSTx")&(substr(objects,5,6)==as.character(val1[i])))
		Amatrix@values[sel,which(objects==paste("der0_",val[i],"sta_x",sep=""))] <- 1
		Amatrix@free[sel,which(objects==paste("der0_",val[i],"sta_x",sep=""))] <- FALSE

		sel <- which((substr(objects,1,4)=="MSTy")&(substr(objects,5,6)==as.character(val1[i])))
		Amatrix@values[sel,which(objects==paste("der0_",val[i],"sta_y",sep=""))] <- 1
		Amatrix@free[sel,which(objects==paste("der0_",val[i],"sta_y",sep=""))] <- FALSE
		}

#____________________________________________________________________________
# Paths from Level of X to Velocity of Y

		diag(Amatrix@free[which((substr(objects,1,4)=="der1")&(substr(objects,11,11)=="x"))[-1],which((substr(objects,1,4)=="der0")&(substr(objects,10,14)=="sta_y"))]) <- TRUE
		diag(Amatrix@labels[which((substr(objects,1,4)=="der1")&(substr(objects,11,11)=="x"))[-1],which((substr(objects,1,4)=="der0")&(substr(objects,10,14)=="sta_y"))]) <- "REG_STy_D1x"

#____________________________________________________________________________
# Estimate Means of Time-Invariant Intercepts
				
	Mmatrix@values[1,which(objects=="traitINT_x")] <- 50
	Mmatrix@labels[1,which(objects=="traitINT_x")] <- "MEAN_traitINT_x"
	Mmatrix@free[1,which(objects=="traitINT_x")] <- TRUE

	Mmatrix@values[1,which(objects=="traitINT_y")] <- 10
	Mmatrix@labels[1,which(objects=="traitINT_y")] <- "MEAN_traitINT_y"
	Mmatrix@free[1,which(objects=="traitINT_y")] <- TRUE

#____________________________________________________________________________
# Creating Initial State Variance and Covariance

	sel1 <- which(objects=="der1_0000_x")
	sel2 <- which(objects=="der0_1000sta_x")
	Amatrix$values[sel2,sel1] <- 1
	sel1 <- which(objects=="der1_0000_y")
	sel2 <- which(objects=="der0_1000sta_y")
	Amatrix$values[sel2,sel1] <- 1

	sel1 <- which(substr(objects,1,10)=="der1_0000_")
	sel2 <- which(substr(objects,1,10)=="der1_0000_")
	Smatrix$labels[sel2,sel1] <- c("P11","P21","P21","P22")
	Smatrix$free[sel2,sel1] <- TRUE
	Smatrix$values[sel2,sel1] <- c(10,0,0,10)

#____________________________________________________________________________
# This duplicates matrices from above for both groups, and assigns unique
# parameter names. Rather than estimate the parameters for both groups, 
# this code estimates the parameters for the first group (Control), and the 
# difference of the second group (Intervention) relative to the control. 
# Consequently, the output provide information about the statistical 
# significance of the group differences. 

	Amatrix1 <- Amatrix2 <- Amatrix; 
	Smatrix1 <- Smatrix2 <- Smatrix; 
	Mmatrix1 <- Mmatrix2 <- Mmatrix; 
	StochasticErr1 <- StochasticErr2 <- StochasticErr

	sel <- !is.na(Amatrix1$labels)
	Amatrix1$labels[sel] <- paste(Amatrix1$labels[sel],"_g1",sep="")
	sel <- (!is.na(Smatrix1$labels))&(substr(Smatrix1$labels,1,7)!="Rescale")
	Smatrix1$labels[sel] <- paste(Smatrix1$labels[sel],"_g1",sep="")
	sel <- !is.na(StochasticErr1$labels)
	StochasticErr1$labels[sel] <- paste(StochasticErr1$labels[sel],"_g1",sep="")
	sel <- !is.na(Mmatrix1$labels)
	Mmatrix1$labels[sel] <- paste(Mmatrix1$labels[sel],"_g1",sep="")

	LabelList <- c("Ax","REG_STy_D1x","Ay","P11","VAR_traitINT_x","P21",
		"P22","COV_traitINTx_INTy","VAR_traitINT_y","MEAN_traitINT_x",
		"MEAN_traitINT_y","VarD1x","CovD1xy","VarD1y")
		
	ParamG1 <- mxMatrix(type = "Full", nrow = 14, ncol = 1,name = "ParamG1", free = T,
		labels = paste(LabelList,"_g1",sep=""),lbound=c(rep(NA,11),0,NA,0))
	ParamG2 <- mxMatrix(type = "Full", nrow = 14, ncol = 1,name = "ParamG2", free = T,
		labels = paste(LabelList,"_g2",sep=""))	
	NewAlg2 <- mxAlgebra(ParamG1+ParamG2, name = "ParamSum")
	
	for(i in 1:3) {
		sel <- Amatrix2$labels==LabelList[i]
		Amatrix2$labels[sel] <- paste("ParamSum[",i,",1]",sep="")
		Amatrix2$free[sel] <- F
		}

	for(i in 4:9) {
		sel <- Smatrix2$labels==LabelList[i]
		Smatrix2$labels[sel] <- paste("ParamSum[",i,",1]",sep="")
		Smatrix2$free[sel] <- F
		}

	for(i in 10:11) {
		sel <- Mmatrix2$labels==LabelList[i]
		Mmatrix2$labels[sel] <- paste("ParamSum[",i,",1]",sep="")
		Mmatrix2$free[sel] <- F
		}
	
	for(i in 12:14) {
		sel <- StochasticErr2$labels==LabelList[i]
		StochasticErr2$labels[sel] <- paste("ParamSum[",i,",1]",sep="")
		StochasticErr2$free[sel] <- F
		}

